!******************************************************************
! Module allocation_mod
!
! Contains various subroutines for numerical procedures.
!******************************************************************
module allocation_mod
    use param
    use cal
    use dal
     
contains
!***************************
! Function AllocateInt8
!***************************
function AllocateInt8(il, iu, name) result(arrayPtr)
    integer, intent(in) :: il, iu
    character(len=*), intent(in) :: name
    integer(kind=int8), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function AllocateInt8

!***************************
! Function AllocateInt32
!***************************
function AllocateInt32(il, iu, name) result(arrayPtr)
    integer, intent(in) :: il, iu
    character(len=*), intent(in) :: name
    integer(kind=int32), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function AllocateInt32

!***************************
! Function Allocate2DInt8
!***************************
function Allocate2DInt8(il, iu, jl, ju, name) result(arrayPtr)
    integer, intent(in) :: il, iu, jl, ju
    character(len=*), intent(in) :: name
    integer(kind=int8), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function Allocate2DInt8


!***************************
! Function Allocate2DInt16
!***************************
function Allocate2DInt16(il, iu, jl, ju, name) result(arrayPtr)
    integer, intent(in) :: il, iu, jl, ju
    character(len=*), intent(in) :: name
    integer(kind=int16), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function Allocate2DInt16

!***************************
! Function Allocate2DInt32
!***************************
function Allocate2DInt32(il, iu, jl, ju, name) result(arrayPtr)
    integer, intent(in) :: il, iu, jl, ju
    character(len=*), intent(in) :: name
    integer(kind=int32), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function Allocate2DInt32

!***************************
! Function AllocateSingle
!***************************
function AllocateSingle(il, iu, name) result(arrayPtr)
    integer, intent(in) :: il, iu
    character(len=*), intent(in) :: name
    real(kind=single), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function AllocateSingle

!***************************
! Function Allocate2DSingle
!***************************
function Allocate2DSingle(il, iu, jl, ju, name) result(arrayPtr)
    integer, intent(in) :: il, iu, jl, ju
    character(len=*), intent(in) :: name
    real(kind=single), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function Allocate2DSingle
!***************************
! Function Allocate3DSingle
!***************************
function Allocate3DSingle(il, iu, jl, ju, kl, ku, name) result(arrayPtr)
    integer, intent(in) :: il, iu, jl, ju, kl, ku
    character(len=*), intent(in) :: name
    real(kind=single), dimension(:, :, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju, kl : ku), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function Allocate3DSingle
!***************************
! Function AllocateDouble
!***************************
function AllocateDouble(il, iu, name) result(arrayPtr)
    integer, intent(in) :: il, iu
    character(len=*), intent(in) :: name
    real(kind=double), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function AllocateDouble

!***************************
! Function Allocate2DDouble
!***************************
function Allocate2DDouble(il, iu, jl, ju, name) result(arrayPtr)
    integer, intent(in) :: il, iu, jl, ju
    character(len=*), intent(in) :: name
    real(kind=double), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function Allocate2DDouble


!***************************
! Function AllocateChar
!***************************
function AllocateChar(length, size, name) result(arrayPtr)
    integer, intent(in) :: length, size
    character(len=*), intent(in) :: name
    character(len=length), pointer, dimension(:) :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(1 : size), stat = status)
    call CheckAllocation(status, name)

end function AllocateChar

!***************************************************
! CheckAllocation
!***************************************************
subroutine CheckAllocation(status, string)
    integer (kind=int32), intent(in) ::  status
    character(len=*), intent(in) :: string
    character(len=100) :: comment
   
    if(status .ne. 0) then
       write(comment, '(a32, a20)') "Could not allocate memory for : ", string 
       write (errstr, *) comment
       call fatal("allocErr") 
    end if
   
end subroutine CheckAllocation
!***************************************************
! CheckDeallocation
!***************************************************
subroutine CheckDeallocation(status, string)
    integer (kind=int32), intent(in) ::  status
    character(len=*), intent(in) :: string
    character(len=100) :: comment
   
    if(status .ne. 0) then
       write(comment, '(a31, a20)') "Failed to release memory for : ", trim(string) 
       write (errstr, *) comment
       call warning("allocErr", comment) 
    end if
   
end subroutine CheckDeallocation 

end module allocation_mod









