! Thursday  7th December version
!+
! Module: omsrclistcomb_mod
!
! Authors:  Christopher Brindle - MSSL
!           Andrew King - MSSL
!           Tim Harries - MSSL
!          
! Description: Combines OSW source list into an observation source list
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!
! History: 
!        98 Feb: created TJH
!        98 Apr 2: changed column names for colours in the table
!                  from eg. B-V to B_V to conform to OGIP FITS
!        98 April 27: changed coord calcs to double (TJH)
!        98 May 14: Added columnExist function in reponse to spr
!        99 May 28: Changed parameter names to conform to new convention.
!        2nd June 00 : Added galactic coordinates as 8 byte data
!        4th June 00 : Modified ra and dec to be 8 byte data
!        4th June 00 : Added memory clean-up at  end  of routine
!        7th July 00 : Ammended code to reflect change of column
!        heading from NOBS to SRC_ID. (SSC-SPR-1911) Low memory failure.
!        5th September 00 : General tidy of the memory allocation and deallocation.	
!        20th September 00 : Replace NULLVALUE by the result of the division 1.0/0.0 
!        This results in the entry NULL in the table which indicates that a ource has
!        not been detected  in a filter.
!        21st September 00 : Added the following columns POSERR,SIGNIFICANCE,FWHM_MAJ & FWHM_MIN
!        21st September 00 : Added in the units for the table
!        27th September 00 : Corrected the call message parameters.
!        2nd November   00 : Range validation of instrumental magnitudes
!        carried out before standard colour calculated using CAL call 
!        24th May 2001 Modified averaging routine so that if any error is zero
!                     the weighting factors are set to 1                
!
!************************************************************************************************
module omsrclistcomb_mod
  use param
  use cal
  use dal
  use Errorhandling
  use sla
  use allocation_mod
  use utility_mod

  implicit none 

  private 

  public omsrclistcomb

  !*****************************
  ! Structure to defines a color
  !*****************************
  type ColourT
     integer :: mag1
     integer :: mag2
     character(len=20) label
     character(len=20) elabel
     character(len=20) flabel
  end type ColourT

contains 
!****************************************
! The main program
!****************************************
subroutine omsrclistcomb

    integer, parameter               :: nFilt = 7             ! no of filters
    integer, parameter               :: nCol = 15             ! no of colours for instrumental magnitudes
    integer, parameter               :: nCol1 = 8             ! no of colours for standard magnitudes
    type(ColourT)                    :: instrumentalColour(nCol)
    type(ColourT)                    :: standardColour(nCol)

    type(DataSetT)                   :: oswDataSet            ! handles
    type(DataSetT)                   :: outDataSet            ! handle for output dataset
    type(TableT)                     :: tableSet
    type(ColumnT) :: pColumn
  
    character(len=10), pointer       :: filter(:)              ! filters for osw source lists
    character(len=1024), pointer     :: oswFile(:)        ! filenames
    character(len=50)                :: label                  ! column label
    integer                          :: nmag                   ! no of mags
    integer                          :: nStart                 ! start no of sources
    integer                          :: nFound                 ! no of identical sources found
    integer(kind=int32), pointer     :: iIndep(:)
    integer                          :: nIndep                 ! number of independent sources
    integer(kind=int32), pointer     :: list(:)                ! list of identical sources
    integer(kind=int32), pointer     :: oswFound(:)
    integer                          :: nosw                   ! no of osw lists
    integer                          :: i,j,k                  ! counters
    integer                          :: nSources               ! total no of sources
    integer(kind=int32), pointer     :: nrows(:)               ! no of rows in each osw source list
    integer                          :: ntemp                  ! temp storage
    integer, allocatable             :: oswtoobs(:,:)          ! cross-links
    integer(kind=int32), pointer     :: oswid(:)               ! no of each osw
    integer(kind=int8),  pointer     :: qflag(:)               ! The source quality flag
    integer(kind=int8),  pointer     :: qflagTable(:, :)       ! Quality flag for each filter
        
    integer(kind=int8),  pointer     :: cflag(:)               ! The source confusion flag
    integer(kind=int8),  pointer     :: cflagTable(:, :)       ! Confusion flag table
      
    integer(kind=int8),  pointer     :: eflag(:)               ! Source extension flag      
    integer(kind=int8),  pointer     :: eflagTable(:, :)       ! Extension flag table
  
    integer(kind=int32), pointer     :: sourceID(:)
    integer :: ndims(1) = (/1/)
    real(kind=double)                :: ra_tmp, dec_tmp         ! tmp storage for coords
    real(kind=single), pointer       :: mag_table(:, :)         ! table of magnitudes
    real(kind=single), pointer       :: magerr_table(:, :)      ! table of magnitude errors
    real(kind=double), pointer       :: distlist(:)            ! list of distances
    real(kind=double)                :: dist                   ! distance between two sources
    real(kind=double)                :: err                    ! error on that distance
    real(kind=single)                :: NullValue              ! Null
    real(kind=single)                :: nsig                   ! tolerance for source id
  


    real(kind=double), pointer       :: ra(:)                  ! source list columns
    real(kind=double), pointer       :: dec(:)


    real(kind=double), pointer       :: ra1(:)                 ! 
    real(kind=double), pointer       :: dec1(:)

                   
    real(kind=single), pointer       :: mag(:)        
    real(kind=single), pointer       :: magerr(:)       
    real(kind=double)                :: weight
    real(kind=double)                :: weightSum
    real(kind=single), pointer       :: poserr(:)   
    real(kind=single), pointer       :: magColour(:)   
    real(kind=single), pointer       :: magColourError(:)   
    real(kind=single), pointer       :: standardMagnitude(:, :)   
    real(kind=single), pointer       :: standardMagnitudeError(:, :)
    real(kind=single), pointer       :: stdMagnitudeStore(:, :, :)   
    real(kind=single), pointer       :: stdMagnitudeErrorStore(:, :, :)
   
    real(kind=single), pointer       :: magnitude(:)   
    real(kind=single), pointer       :: magnitudeError(:)   
    real(kind=single), pointer       :: magTemp(:)   
    real(kind=single), pointer       :: magErrTemp(:)   
 
 

    real(kind=single), pointer       :: significance(:)          ! source list columns
    real(kind=single), pointer       :: significanceTable(:, :)   ! table of magnitudes


    real(kind=single)                :: significanceTemp         ! Temporary storage


    real(kind=single), pointer       :: fullHalfWidthMajor(:)    ! source list columns
    real(kind=single), pointer        :: majorTable(:,:)          ! table of major axes
    real(kind=single)                :: majorTemp                ! Temporary storage
    real(kind=single), pointer       :: fullHalfWidthMinor(:)    ! source list columns
    real(kind=single), pointer       :: positionAngle(:)         ! source position angle
 
    real(kind=single), pointer       :: minorTable(:, :)          ! table of minor axes
    real(kind=single), pointer       :: paTable(:, :)             ! table of position angles

    real(kind=single)                :: minorTemp                ! Temporary storage
    real(kind=single)                :: paTemp
    real(kind=single)                :: poserr_tmp
    real(kind=double), pointer       :: galong(:)                ! array for galactic longitudes
    real(kind=double), pointer       :: galat(:)                 ! array for galactic latitudes    
    real(kind=double)                :: stdColour
    real(kind=double)                :: stdMagnitude1
    real(kind=double)                :: stdMagnitude2
    real(kind=double)                :: stdColourError
    real(kind=double)                :: stdMagnitude1Error
    real(kind=double)                :: stdMagnitude2Error
  
    real(kind=single)                :: mean
    real(kind=single)                :: standardDeviation 
    

 
    real(kind=double), parameter     :: pi = 3.14159265358979323846_double      ! PI
    real(kind=double), parameter     :: degreesToRadians = pi / 180.0_double
    real(kind=double), parameter     :: RadiansToDegrees = 180.0_double / pi
 
    integer                          :: noColours
    character(len=100)               :: colourString(15), stdString(15)
    character(len=100)               :: colourErrorString(15)
    real(kind=single), pointer       :: stdColourMagnitude(: , :)
    real(kind=single), pointer       :: stdColourMagnitudeError(: , :)
    integer(kind=int32), pointer     :: RASortIndex(:)
    integer                          :: iColumn, jColumn      
    integer                          :: noFilters
    integer                          :: filterIndex(20)    
    integer                          :: allocErr
    integer(kind=int32), pointer     :: nCount(:, :)            
    integer(kind=int32), pointer     :: index(:)
    logical, allocatable             :: magexist(:,:)            ! does a magnitude exist?
    logical, dimension(1 : nCol)     :: stdMagExist              ! Does a standard magnitude exist?
  
    character(len=5)                 :: filtid(nFilt) = &
                             (/'WHITE','UVW2 ','UVM2 ','UVW1 ','U    ','B    ','V    '/) 
    integer(kind=int32)              :: filt(7) =  (/ WHITE, UVW2, UVM2, UVW1, U, B, V /)
    integer(kind=int32)               :: intNullValue = 10      
    integer :: iflag
   
    real(kind=double) :: raz, decz, xi, eta

    
    !************************
    ! Iniitialise NullValue
    !************************
    NullValue = -real(99999)
   
    !************************************************
    ! Initialise the filter prority and colour 
    ! tables for the instrumental magnitudes
    !************************************************
    call SetUpInstrumentalColourTable(instrumentalColour)


    !************************************************
    ! Initialise the filter prority and colour 
    ! tables for the standard magnitudes
    !************************************************
    call SetUpStandardColourTable(standardColour)



    !************************************************
    ! Get the number of osw files and their names.
    !************************************************

    nosw = parameterCount("sourcelistsets")


    oswFile => AllocateChar(1024, nosw, "oswFile")

    nsig = realParameter("nsigma")

    nrows => AllocateInt32(1, nosw, "nrows")
 
    filter => AllocateChar(10, nosw, "filter")
    
 
    
    nSources = 0 

  

    !**********************************
    ! Determine total number of sources
    !**********************************

    do i = 1 , nosw
                
        oswfile(i) = stringParameter("sourcelistsets", i - 1)
       
        oswDataSet = DataSet(oswFile(i),read)
      


        tableSet = table(oswDataSet, 0)
        nrows(i) = numberOfRows(tableSet)
        nSources = nSources + nrows(i)
       
        call release(tableSet)
        call release(oswDataSet)

    end do
    !*************************************
    ! Check there is at least 1 source
    !*************************************
    if (nSources .eq. 0) then
        call fatal("Can't continue", "No sources to process")
    end if    

    !******************************************
    ! create the output observation source list
    !******************************************
    call CreateObsSourceList(nOsw, oswFile, outDataSet, raz, decz)    
    raz = raz * degreesToRadians
    decz = decz * degreesToRadians
    !******************************************
    ! Allocate memory for the arrays to hold
    ! the input data.
    !******************************************
    oswid => AllocateInt32(1, nSources, "oswid")


    ra => AllocateDouble(1, nSources, "ra")
    dec => AllocateDouble(1, nSources, "dec")


    significance => AllocateSingle(1, nSources, "significance")
    fullHalfWidthMajor => AllocateSingle(1, nSources, "fullHalfWidthMajor")
    fullHalfWidthMinor => AllocateSingle(1, nSources, "fullHalfWidthMinor")
    positionAngle => AllocateSingle(1, nSources, "positionAngle")
    galong => AllocateDouble(1, nSources, "galong")
    galat => AllocateDouble(1, nSources, "galat")
    poserr => AllocateSingle(1, nSources, "poserr")
    mag => AllocateSingle(1, nSources, "mag")
    magerr => AllocateSingle(1, nSources, "magerr")
    qflag => AllocateInt8(1, nSources, "qflag")
    cflag => AllocateInt8(1, nSources, "cflag")
    eflag => AllocateInt8(1, nSources, "eflag")
 
    allocate(oswtoobs(1 : nSources, 3), stat=allocErr)
    call CheckAllocation(allocErr, "oswtoobs")

    !********************************************************
    ! Read in the data from each osw source list into the 
    ! appropriate array- appending the new data to the end of
    ! that array.
    !********************************************************
      
    nStart = 0

    !**********************************
    ! Determine total number of sources
    !**********************************

 

    do i = 1 , nosw
       oswDataSet = DataSet(oswFile(i), read)
       filter(i) = trim(stringAttribute(oswDataSet, "FILTER"))
       tableSet = table(oswDataSet, 0)

       forall (j = 1  : nrows(i))
          oswid(nStart + j) = i
          oswtoobs(nStart + j, 2) = j
       end forall

       call ReadReal64Column(tableSet, "RA", nrows(i), nStart, ra)
       call ReadReal64Column(tableSet, "DEC", nrows(i), nStart, dec)
       call ReadReal32Column(tableSet, "FWHM_MIN", nrows(i), nStart, fullHalfWidthMinor)
       call ReadReal32Column(tableSet, "FWHM_MAJ", nrows(i), nStart, fullHalfWidthMajor)
       call ReadReal32Column(tableSet, "PA", nrows(i), nStart, positionAngle)
       call ReadReal32Column(tableSet, "POSERR", nrows(i), nStart, poserr)
       call ReadReal32Column(tableSet, "SIGNIFICANCE", nrows(i), nStart, significance)
       call ReadReal32Column(tableSet, "MAG", nrows(i), nStart, mag)
       call ReadReal32Column(tableSet, "MAGERR", nrows(i), nStart, magerr)
       
       call ReadInt8Column  (tableSet, "QFLAG", nrows(i), nStart, qflag)
       call ReadInt8Column  (tableSet, "CFLAG", nrows(i), nStart, cflag)
       call ReadInt8Column  (tableSet, "EFLAG", nrows(i), nStart, eflag)


       call release(tableSet)
       call release(oswDataSet)
       nStart = nStart + nrows(i)

    end do
    where (mag .gt. 100) 
        mag = NullValue
        magerr = NullValue
    end where
  
    !***************************************
    ! Do a few conversions
    !***************************************
    ra = ra * degreesToRadians
    dec = dec * degreesToRadians

    !***************************************
    ! Allocate memory for output arrays.
    !***************************************
    list => AllocateInt32(1, nSources, "list")
    oswFound => AllocateInt32(1, nSources, "oswFound")
    distlist => AllocateDouble(1, nSources, "distlist")
    mag_table => Allocate2DSingle(1, nSources, 1, nFilt, "mag_table")
    magerr_table => Allocate2DSingle(1, nSources, 1, nFilt, "magerr_table")
    significanceTable => Allocate2DSingle(1, nSources, 1, nFilt, "significanceTable")
    majorTable => Allocate2DSingle(1, nSources, 1, nFilt, "majorTable")
    minorTable => Allocate2DSingle(1, nSources, 1, nFilt, "minorTable")
    paTable => Allocate2DSingle(1, nSources, 1, nFilt, "paTable")
    qflagTable => Allocate2DInt8(1, nSources, 1, nFilt, "qflagTable")
    cflagTable => Allocate2DInt8(1, nSources, 1, nFilt, "cflagTable")
    eflagTable => Allocate2DInt8(1, nSources, 1, nFilt, "eflagTable")


    allocate(magexist(1 : nSources, 1 : nFilt), stat=allocErr)
    call CheckAllocation(allocErr, "magExist")

  
    mag_table = NullValue
    magerr_table = NullValue
    significanceTable = NullValue
    majorTable = NullValue
    minorTable = NullValue
    paTable = NullValue
    qflagTable = intNullValue
    cflagTable = intNullValue
    eflagTable = intNullValue
  

    magexist = .false.
    magTemp => AllocateSingle(1, nSources, "magTemp")
    magErrTemp => AllocateSingle(1, nSources, "magErrTemp")
 
 
    !******************************************************
    ! Find how many identical sources there are in 2 or more
    ! of the osw files. 
    !******************************************************

    do i = 1 , nSources

       nFound = 0

       !**********************************************************
       ! if oswid(n) <= 0 then then we have found the counterparts
       !  of this source- otherwise this source is the first
       !**********************************************************
       if (oswid(i) .gt. 0) then
           nFound = 1
           list(nFound) = i
           distlist(nFound) = 0.0_double
       end if
       !******************************************
       ! now loop over all the rest of the sources
       !******************************************

       do j = i + 1, nSources
 
          !************************************************
          ! make sure this source hasn't been found already 
          !************************************************
          if ((oswid(i) .ne. oswid(j)) .and. (oswid(i) .gt. 0) .and. (oswid(j) .gt. 0)) then

             !*************************************
             ! calculate the distance to the source
             ! and its error (both in arcsecs)
             !*************************************
             dist = ArcLength(ra(i), dec(i), ra(j), dec(j))
             err = dble(nsig) * AddErrors(dble(poserr(i)), dble(poserr(j)))
             err= min(5.0_double, err)      
             !****************************************************** 
             ! if the distance is less than the error then these are 
             ! (might be) the same source
             !******************************************************
             if (dist .lt. err) then  ! nsig is a task parameter
                nFound = nFound + 1
                list(nFound) = j
                distlist(nFound) = dist        ! add it to the list
                oswFound(nFound) = oswid(j)
             end if
          end if
       end do

       j = 1

       do while(j .lt. nFound)
          if (oswFound(j) .eq. oswFound(j + 1)) then
             if (distlist(j + 1) .gt. distlist(j)) then
                if (j + 1 .eq. nFound) then
                   nFound = nFound - 1
                else
                   do k = j + 1, nFound - 2
                      list(k) = list(k+1)
                      distlist(k) = distlist(k + 1)
                      oswFound(k) = oswFound(k + 1)
                   end do
                   nFound = nFound - 1
                end if
             end if

          end if
          j = j + 1
       end do

       j = 1
       do while(j .lt. nFound)
          if (oswFound(j) .eq. oswFound(j + 1)) then
             if (j + 1 .eq. nFound) then
                nFound = nFound - 1
             else
                do k = j + 1, nFound - 2
                   list(k) = list(k + 1)
                   distlist(k) = distlist(k + 1)
                   oswFound(k) = oswFound(k + 1)
                end do
                nFound = nFound - 1
             end if
          else
             j = j + 1
          end if
       end do

       if (nFound .gt. 0) then
          write(errstr, *) "Same sources: ", list(1 : nFound)

          call message(AppMsg, Verbose)
          write(errstr, *) "distances: ", real(distlist(1 : nFound))

          call message(AppMsg, Verbose)
 
          !*************************************** 
          ! find the weighted mean of the position
          !***************************************      

          ra_tmp = 0.0_double
          dec_tmp = 0.0_double
          poserr_tmp = 0.0_double
          weightSum = 0.0

          do k = 1,  nFound
             if (poserr(list(k)) .gt. 0.0) then
                 weight = 1.0_double / (poserr(list(k)) * poserr(list(k)))
             else
                 weight = 1.0
             end if    
             weightSum = weightSum + weight
             !*************************************************************
             ! Conert RAs and decs to tangent-plane coordinates- average
             ! the sum of these and then convert the average to RA and dec
             !*************************************************************

             call sla_ds2tp(ra(list(k)), dec(list(k)), raz, decz, xi, eta, iflag)

             ra_tmp = ra_tmp + xi * weight     
             dec_tmp = dec_tmp + eta * weight
             poserr_tmp = poserr_tmp + poserr(list(k)) * weight
                       
          end do

          ra_tmp = ra_tmp / weightSum
          dec_tmp = dec_tmp / weightSum
          call sla_dtp2s(ra_tmp, dec_tmp, raz, decz, ra_tmp, dec_tmp)
          poserr(i) = poserr_tmp / weightSum
          
          !*****************************************************
          ! Go through and fill up the magnitude matrix
          !  
          ! loop over all the filters
          !******************************************************
          do j = 1 , nFilt

             nmag = 0
             significanceTemp = 0.0
             minorTemp = 0.0
             majorTemp = 0.0
             paTemp = 0.0
             !*********************************
             ! loop over all identical sources
             !*********************************
             do k = 1 , nFound

                if ( trim(filter(oswid(list(k)))) .eq. trim(filtid(j)) ) then

                     
                   nmag = nmag + 1      
                   magTemp(nmag) = mag(list(k))
                   magErrTemp(nmag) = magerr(list(k))

                   significanceTemp = significanceTemp + significance(list(k))
                   majorTemp = majorTemp + fullHalfWidthMajor(list(k))
                   minorTemp = minorTemp + fullHalfWidthMinor(list(k))
                   paTemp = paTemp + positionAngle(list(k))
               end if

             end do
              
             !**********************************
             ! Compute the average values
             !**********************************
             if (nmag .gt. 0) then

                call ComputeAverageMagnitude(magTemp, magErrTemp, nmag, mean, standardDeviation)        
                mag_table(i, j) = mean  
                magerr_table(i, j) = standardDeviation

                significanceTable(i, j) = significanceTemp / real(nmag)

                majorTable(i, j) = majorTemp / real(nmag)
                minorTable(i, j) = minorTemp / real(nmag)
                paTable(i, j)   = paTemp / real(nmag)
                magexist(i, j) = .true.
                qflagTable(i, j) = qflag(i) 
                cflagTable(i, j) = cflag(i)
                eflagTable(i, j) = eflag(i) 
             end if

          end do

          !*************************************************
          ! Need to remember where osw each  source came from
          !*************************************************
          forall (k = 1  : nFound)
             oswtoobs(list(k), 3) = list(1)
             oswid(list(k)) = 0         ! take the source out of the running
          end forall

          oswid(list(1)) = -1     ! we flag the first source found 
       end if

    end do



    deallocate(magTemp, stat=allocErr)
    call CheckDeallocation(allocErr, "magTemp")

    deallocate(magErrTemp, stat=allocErr)
    call CheckDeallocation(allocErr, "magErrTemp")

    deallocate(list, stat=allocErr)
    call CheckDeallocation(allocErr, "list")

    deallocate(distList, stat=allocErr)
    call CheckDeallocation(allocErr, "distList")
 
    deallocate(oswFound, stat=allocErr)
    call CheckDeallocation(allocErr, "oswFound")

    deallocate(filter, stat=allocErr)
    call CheckDeallocation(allocErr, "filter")

    deallocate(qflag, stat=allocErr)
    call CheckDeallocation(allocErr, "qflag")


    deallocate(cflag, stat=allocErr)
    call CheckDeallocation(allocErr, "cflag")


    deallocate(eflag, stat=allocErr)
    call CheckDeallocation(allocErr, "eflag")

    iIndep => AllocateInt32(1, nSources, "iIndep")
    !*******************************************
    ! Compute the number of independent sources
    ! i.e. those with a oswid(n) = -1
    ! and reorder the arrays.
    !********************************************
    ntemp = 0
    do i = 1 , nSources

       if (oswid(i) .lt. 0) then
          ntemp = ntemp + 1
      
          ra(ntemp) = ra(i)
          dec(ntemp) = dec(i)
          fullHalfWidthMinor(ntemp) =  fullHalfWidthMinor(i)
          fullHalfWidthMajor(ntemp) =  fullHalfWidthMajor(i)
          positionAngle(ntemp) =  positionAngle(i)
          
          poserr(ntemp) = poserr(i)
          significance(ntemp) = significance(i)
          
          forall (j = 1 : nFilt)
              mag_table(ntemp, j) = mag_table(i, j)
              magerr_table(ntemp, j) = magerr_table(i, j)
              significanceTable(ntemp, j) = significanceTable(i, j)
              majorTable(ntemp, j) = majorTable(i, j)
              minorTable(ntemp, j) = minorTable(i, j)
              paTable(ntemp, j) = paTable(i, j)
              qflagTable(nTemp, j) = qflagTable(i, j)
              cflagTable(nTemp, j) = cflagTable(i, j)
              eflagTable(nTemp, j) = eflagTable(i, j)

              magexist(ntemp, j) = magexist(i, j)
          end forall           

       end if
       iIndep(i) = ntemp
    end do

    nIndep = ntemp

    write(errstr,'(i6, a)') ntemp,' independent sources'
    call message(AppMsg, Verbose)

    deallocate(oswid, stat=allocErr)
    call CheckDeallocation(allocErr, "oswid")

    deallocate(fullHalfWidthMinor, stat=allocErr)
    call CheckDeallocation(allocErr, "fullHalfWidthMinor")
    
    deallocate(fullHalfWidthMajor, stat=allocErr)
    call CheckDeallocation(allocErr, "fullHalfWidthMajor")
   
    deallocate(positionAngle, stat=allocErr)
    call CheckDeallocation(allocErr, "positionAngle")
  
    !**************************************************
    ! Create the observation source list
    ! which has nIndep rows
    !**************************************************
    tableSet = addTable(outDataSet, "SRCLIST", nIndep)

    RASortIndex => AllocateInt32(1, nSources, "RASortIndex")
 
    forall (i = 1 : nIndep) RASortIndex(i) = i
   
    call AddInt32IndexColumn(tableSet, "SRCID", "", nIndep, RASortIndex, RASortIndex)
 
    !*****************************************************
    ! Find the order of increasing  RA- store in RASortIndex
    ! for adding data to source list table in  
    ! increasing RA order (ra is not sorted)
    !****************************************************
    call HeapSort(ra, RASortIndex, nIndep)

    !*************************************************** 
    ! Compute the galactic longitude and latitude 
    ! from the averaged RA and dec
    !***************************************************
    do i = 1, nIndep
        call sla_EQGAL(ra(i), dec(i), galong(i), galat(i)) 
    end do   



    !********************************************
    ! Add the refined ra's and dec's to the table 
    !********************************************
    ra1 => AllocateDouble(1, nIndep, "ra1")
    dec1 => AllocateDouble(1, nSources, "dec1")

    forall (i = 1 : nIndep) 
        ra1(i) = ra(i)
        dec1(i) = dec(i)
    end forall



    ra = ra * radiansToDegrees
    call AddReal64IndexColumn(tableSet, "RA", "degrees", nIndep, ra, RASortIndex, nullValue)
    dec = dec * radiansToDegrees
    call AddReal64IndexColumn(tableSet, "DEC", "degrees", nIndep, dec, RASortIndex, nullValue)



    deallocate(ra, stat = allocErr)
    call CheckDeallocation(allocErr, "ra")
  
    deallocate(dec, stat = allocErr)
    call CheckDeallocation(allocErr, "dec")
 

    deallocate(ra1, stat = allocErr)
    call CheckDeallocation(allocErr, "ra1")
  

    deallocate(dec1, stat = allocErr)
    call CheckDeallocation(allocErr, "dec1")
   
    call AddReal32IndexColumn(tableSet, "POSERR", "arcsec", nIndep, poserr, RASortIndex, nullValue)
    deallocate(poserr, stat = allocErr)
    call CheckDeallocation(allocErr, "poserr")

    galong = galong * radiansToDegrees
    call AddReal64IndexColumn(tableSet, "LII", "degrees", nIndep, galong, RASortIndex, nullValue)
    deallocate(galong, stat = allocErr)
    call CheckDeallocation(allocErr, "galong")
 

    galat = galat * radiansToDegrees
    call AddReal64IndexColumn(tableSet, "BII", "degrees", nIndep, galat, RASortIndex, nullValue)
    deallocate(galat, stat = allocErr)
    call CheckDeallocation(allocErr, "galat")
    
    !**************************************
    ! Find out what filters have magnitudes
    !************************************** 
    noFilters = 0
    do i = 1 , nFilt
   
        if (ANY(magexist(1 : nIndep, i : i)))  then
            noFilters = noFilters + 1
            filterIndex(noFilters) = i
        else
        
            write(errstr,*) "No sources found in filter ", filtid(i)
            call message(AppMsg, Verbose)

        end if

    end do
 
    !******************************************************
    ! Add the significance arrays
    !****************************************************** 
    do j = 1 , noFilters
       i = filterIndex(j)
       
       label = trim(filtid(i)) // "_SIGNIF"
       call Add2Real32IndexColumn(tableSet, label, "sigma", nIndep, significanceTable, i, RASortIndex, nullValue)
    end do
 
    deallocate(significance, stat=allocErr)
    call CheckDeallocation(allocErr, "significance")

    deallocate(significanceTable, stat=allocErr)
    call CheckDeallocation(allocErr, "significanceTable")
   


    !*********************************************
    ! Add the instrumental magnitude and its error
    !*********************************************
    do j  = 1 , noFilters
        i = filterIndex(j)

        label = trim(filtid(i)) // "_INS"
        call Add2Real32IndexColumn(tableSet, label, "magnitudes", nIndep, mag_table, i, RASortIndex, nullValue)

        label = "ERR_" // trim(filtid(i)) // "_INS"
        call Add2Real32IndexColumn(tableSet, label, "magnitudes", nIndep, magerr_table, i, RASortIndex, nullValue)
    end do

    !*********************************************
    ! Add the instrumental colours
    !*********************************************
    magColour => AllocateSingle(1, nIndep, "magColour")
    magColourError => AllocateSingle(1, nIndep, "magColourError")

    do i = 1, nCol     ! Loop through all the possible colours
       magColour = NullValue
       magColourError = NullValue
       iColumn = instrumentalColour(i)%mag1
       jColumn = instrumentalColour(i)%mag2

       if (ANY(magexist(1 : nIndep, iColumn : iColumn)) .and.  &
            ANY(magexist(1 : nIndep, jColumn : jColumn))) then

          do k = 1, nIndep
              if (mag_table(k, iColumn) .ne. NullValue .and. &
                  mag_table(k, jColumn) .ne. NullValue) then
                  magColour(k) = mag_table(k, iColumn) - mag_table(k, jColumn)
                  magColourError(k) = AddErrors(dble(magerr_table(k, iColumn)), dble(magerr_table(k, jColumn)))
              end if
          end do 

          label = trim(instrumentalColour(i)%label) // "_INS"
          
          call AddReal32IndexColumn(tableSet, label, "magnitudes", nIndep, magColour, RASortIndex, nullValue)

          label = trim(instrumentalColour(i)%elabel) // "_INS"
          call AddReal32IndexColumn(tableSet, label, "magnitudes", nIndep, magColourError, RASortIndex, nullValue)

       end if
    end do

    !*******************************************************************
    ! now we have the colours
    ! we can currently have (U-B) and/or (B-V), but the combinations
    ! can be added straightfowardly by changing the array in SetUpColours
    !********************************************************************
    call CAL_setState(OM , 1, accuracyLevel = ACCURACY_LOW)

    !****************************************
    ! Allocate memory for arrays holding
    ! the standard colours
    !*************************************** 
    stdColourMagnitude => Allocate2DSingle(1, nIndep, 1, ncol, "stdColourMagnitude")
    stdColourMagnitude = NullValue  

    stdColourMagnitudeError => Allocate2DSingle(1, nIndep, 1, ncol, "stdColourMagnitudeError")
    stdColourMagnitudeError = NullValue  

    !***************************************************
    ! Calculate the standard magnitudes and colours
    ! NB- can only use U, B and V  
    !***************************************************
    standardMagnitude => Allocate2DSingle(1, nIndep, 1, nCol, "standardMagnitude")
    standardMagnitude = NullValue

    standardMagnitudeError => Allocate2DSingle(1, nIndep, 1, nCol, "standardMagnitudeError")
    standardMagnitudeError = NullValue    

    stdMagExist = .false.
    noColours = 0

    !*****************************************************************
    ! Calculate the standard colours and magnitudes in reverse order 
    ! to preserve the priority
    ! Store the different standard magnitudes and errors for each source
    ! that have been computed for each colour combination (then average
    ! them after and calculate the colours  
    !*****************************************************************

    nCount => Allocate2DInt32(1, nCol, 1, nCol, "nCount") 
    nCount = 0
    stdMagnitudeStore => Allocate3DSingle(1, nIndep, 1, nCol, 1, nCol,   "stdMagnitudeStore")
    stdMagnitudeErrorStore => Allocate3DSingle(1, nIndep, 1, nCol, 1, nCol,   "stdMagnitudeErrorStore")
   
    stdMagnitudeStore = NullValue
    stdMagnitudeErrorStore = NullValue


    nCount = 0

    do i = nCol1, 1, -1
      
        iColumn = standardColour(i)%mag1
        jColumn = standardColour(i)%mag2

        if (ANY(magexist(1 : nIndep, iColumn : iColumn)) .and.  &
            ANY(magexist(1 : nIndep, jColumn : jColumn))) then
            noColours = noColours + 1
            colourString(noColours) = standardColour(i)%label
            colourErrorString(noColours) = standardColour(i)%elabel
            stdString(noColours) = standardColour(i)%flabel
            
            stdMagExist(iColumn) = .true.
            stdMagExist(jColumn) = .true.
          
            do k = 1, nIndep
                if (mag_table(k, iColumn) .ne. NullValue .and. &
                    mag_table(k, jColumn) .ne. NullValue) then

                    !************************************************
                    ! Calculate the standard magnitudes and colour
                    !************************************************
                    call CalculateStandardColour(filt(iColumn), filt(jColumn),        &
                        dble(mag_table(k, iColumn)),  dble(magerr_table(k, iColumn)), &
                        dble(mag_table(k, jColumn)),  dble(magerr_table(k, jColumn)),    &
                        stdMagnitude1, stdMagnitude1Error,                            &
                        stdMagnitude2, stdMagnitude2Error,                            &
                        stdColour, stdColourError)
                    !**************************************************************
                    ! Set nCount to 1 to record that this colour was used to
                    ! compute a standard colour
                    !************************************************************** 
                    nCount(iColumn, noColours) = 1
                    nCount(jColumn, noColours) = 1
                
                    stdMagnitudeStore(k, iColumn, noColours) = stdMagnitude1
                    stdMagnitudeErrorStore(k, iColumn, noColours) = stdMagnitude1Error 
                    stdMagnitudeStore(k, jColumn, noColours) = stdMagnitude2
                    stdMagnitudeErrorStore(k, jColumn, noColours) = stdMagnitude2Error
                    stdColourMagnitude(k, noColours) = stdColour
                    stdColourMagnitudeError(k, noColours) = stdColourError
    
                end if
            end do 

        end if
    end do
  
    deallocate(magexist, stat=allocErr)
    call CheckDeallocation(allocErr, "magexist")

    deallocate(mag_table,stat=allocErr)
    call CheckDeallocation(allocErr, "mag_table")
    
    deallocate(magerr_table,stat=allocErr)
    call CheckDeallocation(allocErr, "magerr_table")
   
    magnitude => AllocateSingle(1, nIndep, "magnitude")
    magnitude = NullValue
    magnitudeError => AllocateSingle(1, nIndep, "magnitudeError")
    magnitudeError = NullValue
     

    !******************************************************
    ! Add the standard magnitude and its error to the table
    ! 
    !******************************************************
    do i = 1 , nFilt

        if (stdMagExist(i))  then
            
            do j = 1, noColours
                if (nCount(i, j) .gt. 0) then
                    magnitude = NullValue
                    magnitudeError = NULLValue
                    forall (k = 1 : nIndep)
                        magnitude(k) = stdMagnitudeStore(k, i, j)
                        magnitudeError(k) = stdMagnitudeErrorStore(k, i, j)
                    end forall

                    label = trim(filtid(i)) // "_STD" // trim(stdString(j))

    
                    call AddReal32IndexColumn(tableSet, label, "magnitudes", nIndep, magnitude, RASortIndex, nullValue)

                    label = "ERR_" // trim(filtid(i)) // "_STD" // trim(stdString(j))
                    call AddReal32IndexColumn(tableSet, label, "magnitudes", nIndep, magnitudeError, RASortIndex, nullValue)
                end if
            end do
        end if
    end do
 


    !********************************************
    ! Add the standard colours to the table
    !********************************************
    do i = 1, noColours
        
        label =  trim(colourString(i)) // "_STD"
        call Add2Real32IndexColumn(tableSet, label, "magnitudes", nIndep, stdColourMagnitude, i, RASortIndex, nullValue)
        label = trim(ColourErrorString(i)) // "_STD"
        call Add2Real32IndexColumn(tableSet, label, "magnitudes", nIndep, stdColourMagnitudeError, i, RASortIndex, nullValue)
    end do    
  

    deallocate(stdMagnitudeStore, stat = allocErr)
    call CheckDeallocation(allocErr, "stdMagnitudeStore")
    deallocate(stdMagnitudeErrorStore, stat = allocErr)
    call CheckDeallocation(allocErr, "stdMagnitudeErrorStore")
    deallocate(magnitude, stat = allocErr)
    call CheckDeallocation(allocErr, "magnitude")
    deallocate(magnitudeError, stat = allocErr)
    call CheckDeallocation(allocErr, "magnitudeError")
     

    deallocate(magColour, stat = allocErr)
    call CheckDeallocation(allocErr, "magColour")
    deallocate(magColourError, stat = allocErr)
    call CheckDeallocation(allocErr, "magColourError")
 
    deallocate(standardMagnitude, stat = allocErr)
    call CheckDeallocation(allocErr, "standardMagnitude")
  
    deallocate(standardMagnitudeError, stat = allocErr)
    call CheckDeallocation(allocErr, "standardMagnitudeError")

    deallocate(stdColourMagnitude, stat=allocErr)
    call CheckDeallocation(allocErr, "stdColourMagnitude")
    
    deallocate(stdColourMagnitudeError, stat=allocErr)
    call CheckDeallocation(allocErr, "stdColourMagnitudeError")
   
    !*************************************
    ! Add the major-axes for each filter 
    !*************************************
    do j = 1 , noFilters
 
       i = filterIndex(j)
       label = trim(filtid(i)) // "_MAJ"       
       call Add2Real32IndexColumn(tableSet, label, "pixels", nIndep, majorTable, i, RASortIndex, nullValue)
    end do

    deallocate(majorTable, stat=allocErr)
    call CheckDeallocation(allocErr, "majorTable")

    !*************************************
    ! Add the minor-axes for each filter 
    !*************************************
    do j = 1 , noFilters

        i = filterIndex(j)
        label = trim(filtid(i)) // "_MIN"
        call Add2Real32IndexColumn(tableSet, label, "pixels", nIndep, minorTable, i, RASortIndex, nullValue)

    end do

    deallocate(minorTable, stat=allocErr)
    call CheckDeallocation(allocErr, "minorTable")


    !*************************************
    ! Add the position-angle for each filter 
    !*************************************
    do j = 1 , noFilters

        i = filterIndex(j)
        label = trim(filtid(i)) // "_PA"
        call Add2Real32IndexColumn(tableSet, label, "degrees", nIndep, paTable, i, RASortIndex, nullValue)

    end do


    deallocate(paTable, stat=allocErr)
    call CheckDeallocation(allocErr, "paTable")


    !*************************************
    ! Add the flag QFLAG
    !*************************************
    do j = 1 , noFilters
 
        i = filterIndex(j)
        label = trim(filtid(i)) // "_QFLAG"
        call Add2Int8IndexColumn(tableSet, label, "none", nIndep, qflagTable, i, RASortIndex, intNullValue)
    end do
   
    deallocate(qflagTable, stat = allocErr)
    call CheckDeallocation(allocErr, "qflagTable")
    
    !*************************************
    ! Add the flag CFLAG
    !*************************************
    do j = 1, noFilters

        i = filterIndex(j)
        label = trim(filtid(i)) // "_CFLAG"
        call Add2Int8IndexColumn(tableSet, label, "none", nIndep, cflagTable, i, RASortIndex, intNullValue)
     
    end do
    deallocate(cflagTable, stat=allocErr)
    call CheckDeallocation(allocErr, "cflagTable")

    !*************************************
    ! Add the flag EFLAG
    !*************************************
    do j = 1, noFilters

        i = filterIndex(j)
        label = trim(filtid(i)) // "_EFLAG"
        call Add2Int8IndexColumn(tableSet, label, "none", nIndep, eflagTable, i, RASortIndex, intNullValue)
  
    end do

    deallocate(eflagTable,stat=allocErr)
    call CheckDeallocation(allocErr, "eflagTable")
  
    call release(tableSet)
    call release(outDataSet)

    !**********************************************************
    ! Add a source ID column to each of the osw source lists
    ! telling us which source went to which entry in the
    ! observation source list 
    !**********************************************************

    nStart = 1
 
    index => AllocateInt32(1, nIndep, "index")
    forall (i = 1 : nIndep) index(RASortIndex(i)) = i

    ndims(1) = 1
    do i = 1 , nosw
        oswDataSet = DataSet(oswFile(i), modify)
        tableSet = table(oswDataSet, 0)
   
        if (hasColumn(tableSet, "SRC_ID")) then 
            call warning("colExist", "SRC)ID column exists in table")
            pcolumn = column(tableSet, "SRC_ID", modify)
        else
            pcolumn = addColumn(tableSet, "SRC_ID", integer32, " ", ndims)
        endif

        sourceID => int32Data(pcolumn)
        do j = nStart, nStart + nrows(i) - 1
            sourceID(j - nStart) = index(iIndep(oswtoobs(j, 3)))

        enddo
        call release(pcolumn)
      
        call release(tableSet)
        call release(oswDataSet)
        nStart = nStart + nrows(i)
    end do


    deallocate(index, stat = allocErr)
    call CheckDeAllocation(allocErr, "index")
  

    deallocate(oswfile, stat=AllocErr)
    call CheckDeallocation(allocErr, "oswfile")
 
    deallocate(RASortIndex, stat = allocErr)
    call CheckDeallocation(allocErr, "RASortIndex")


    deallocate(nrows, stat=allocErr)
    call CheckDeallocation(allocErr, "nrows")
    
    deallocate(oswtoobs, stat=allocErr)
    call CheckDeallocation(allocErr, "oswtoobs")

    deallocate(iIndep, stat=allocErr)
    call CheckDeallocation(allocErr, "iIndep")
    
end subroutine omsrclistcomb
!****************************************************
! CalculateStandardColour - Given the two filters, two
! instrumental magnitudes and colours, calculates the 
! standard colours. If the instrumental colours are outside
! the validity range the values are set to nullValue.
!*****************************************************
subroutine CalculateStandardColour(filterID1, filterID2, &
           instrumentalMagnitude1, instrumentalMagnitude1Error, & 
           instrumentalMagnitude2, instrumentalMagnitude2Error, &
           standardMagnitude1, standardMagnitude1Error, & 
           standardMagnitude2, standardMagnitude2Error, &
           standardColour, standardColourError)
  
    integer(kind=int32) :: filterID1, filterID2
    real(kind=double), intent(in) :: instrumentalMagnitude1, instrumentalMagnitude1Error
    real(kind=double), intent(in) :: instrumentalMagnitude2, instrumentalMagnitude2Error
            
    real(kind=double), intent(out) :: standardMagnitude1, standardMagnitude2                     
    real(kind=double), intent(out) :: standardColour
    real(kind=double), intent(out) :: standardColourError
    real(kind=double), intent(out) :: standardMagnitude1Error
    real(kind=double), intent(out) :: standardMagnitude2Error  
    real(kind=double)    :: magnitudeRange
    type(OmColorTransformatorT)     :: colourTrans
    type(IntervalT)                 :: colourValInt


    
    colourTrans = CAL_omGetColorTransformator(filterID1, filterID2)
    colourValInt = CAL_omColorTransValidityRange(colourTrans)  


    !*******************************************************
    ! Check that the instrumental magnitudes lie within
    ! the allowed range.
    !*******************************************************
    magnitudeRange = instrumentalMagnitude1 - instrumentalMagnitude2

    if (magnitudeRange .gt. colourValInt%lower .and. &
        magnitudeRange .lt. colourValInt%upper)  then    ! Range okay                       
       
        !****************************************************
        ! NB The first magnitude in the following call is
        ! transformed into the standard system
        !****************************************************
        call CAL_omColorTransform(filterId1, filterID2, &
                  instrumentalMagnitude1, instrumentalMagnitude2, &
                  standardColour,  standardMagnitude2, &
                  standardColourError, standardMagnitude2Error)


        standardMagnitude1Error = AddErrors(standardMagnitude2Error,  standardColourError) 

!***************************************************************************
! Temporary (18.2.2002) Because the cal routine does not yet calculate the 
! standard magnitude errors properly, just set the error equal to
! the instrumental magnitude error
!***************************************************************************  

        standardMagnitude1Error = instrumentalMagnitude1Error
        standardMagnitude2Error = instrumentalMagnitude2Error


        standardMagnitude1 = standardMagnitude2 + standardColour

    end if 
                                           
end subroutine CalculateStandardColour
!******************************************
! CreateObsSourceList
! create the output observation source list
!******************************************
subroutine CreateObsSourceList(noOswFiles, oswFile, outDataSet, raPoint, decPoint)
    type(DataSetT), intent(out) :: outDataSet
    integer, intent(in) :: noOswFiles
    character(len=*), intent(in), dimension(1 : noOswFiles) :: oswFile
    real(kind=double), intent(out) :: raPoint, decPoint
    character(len=40) :: startTimeString, endTimeString
    type(DataSetT) :: oswDataSet
    integer i, k
    character(len=40) :: object, observer, obsID, startObs, endObs
        

    outDataSet = DataSet(stringParameter("outset"), create)
  
    call setAttribute(outDataSet, "HDUCLASS", "OGIP", "Format conforms to OGIP/GSFC conventions")
    call setAttribute(outDataSet, "HDUCLAS1", "SRCLIST", "File contains a source list")
    call setAttribute(outDataSet, "FILTER", "n/a", "Not applicable")
    call setAttribute(outDataSet, "CONTENT", "SRCLIST", "File content")



    !************************************************
    ! 21.2.2002 - Add the new attributes for SCR 84
    !************************************************
    call OAL_addCommonAttributes(outDataSet)



    !************************************************
    ! Find the first start time and the last end time
    !************************************************
    oswDataSet = DataSet(oswFile(1), read)

    startTimeString = stringAttribute(oswDataSet, "DATE-OBS")
    endTimeString   = stringAttribute(oswDataSet, "DATE-END")
    obsID   = stringAttribute(oswDataSet, "OBS_ID")
    raPoint = real64Attribute(oswDataSet, "RA_PNT")
    decPoint = real64Attribute(oswDataSet, "DEC_PNT")
    object   = trim(stringAttribute(oswDataSet, "OBJECT"))
    observer   = trim(stringAttribute(oswDataSet, "OBSERVER"))
    
    call release(oswDataSet)

    do i = 2, noOswFiles
 
        oswDataSet = DataSet(oswFile(i), read)
        startObs = stringAttribute(oswDataSet, "DATE-OBS")
        endObs   = stringAttribute(oswDataSet, "DATE-END")
        if (trim(stringAttribute(oswDataSet, "OBJECT")) .ne. object) call fatal("wrong object")
        


        !******************************************************
        ! Need the camera pointing direction to do the 
        ! astrometry correction
        !***************************************************** 
 
        raPoint = raPoint + real64Attribute(oswDataSet, "RA_PNT")
        decPoint = decPoint + real64Attribute(oswDataSet, "DEC_PNT")
      


        k = CompareDates(startTimeString, startObs)
        
        if (k .lt. 0) startTimeString = startObs

        k = CompareDates(endTimeString, endObs)
        
        if (k .gt. 0) endTimeString = endObs
          
        call release(oswDataSet)  
    end do
    raPoint = raPoint / dble(noOswFiles)
    decPoint = decPoint / dble(noOswFiles)

    call setAttribute(outDataSet, "OBS_ID", obsID, "Observation identifier")
    call setAttribute(outDataSet, "OBJECT", object, "Object")
    call setAttribute(outDataSet, "OBSERVER", observer, "Observer")
 
    call setAttribute(outDataSet, "DATE-OBS", startTimeString, "Start time of observations")
    call setAttribute(outDataSet, "DATE-END", endTimeString, "End time of observations")
    call setAttribute(outDataSet, "RA_PNT", raPoint, "degrees", "Averaged RA pointing direction")
    call setAttribute(outDataSet, "DEC_PNT", decPoint, "degrees", "Averaged Dec pointing direction")
       

end subroutine CreateObsSourceList
!*******************************************************
! AddErrors
!
!*******************************************************
real(kind=double) function AddErrors(error1, error2)
    real(kind=double), intent(in) :: error1, error2
 
    AddErrors = dsqrt(error1 * error1  + error2 * error2)

end function AddErrors
!******************************************************
! subroutine SetUpInstrumentalColourTable
! Initialises the array with the possible colours
! NB Filter codes are V=V, U=U, B=B, White=W, Grism2=G,
! UVW1=L, UVM2=M, UVW2=S, Grism1=H.
!******************************************************
subroutine SetUpInstrumentalColourTable(Colour)

    type(ColourT), intent(out), dimension(:) :: Colour
   
    !***************** 
    ! These are OPT-OPT
    !*****************
    Colour(1) = ColourT(5, 6, "U_B", "ERR_U_B", "")
    Colour(2) = ColourT(6, 7, "B_V", "ERR_B_V", "")
    Colour(3) = ColourT(5, 7, "U_V", "ERR_U_V", "")

    !****************
    ! These are UV-UV
    !****************
    Colour(4) = ColourT(2, 4, "S_L", "ERR_S_L", "")
    Colour(5) = ColourT(2, 3, "S_M", "ERR_S_M", "")
    Colour(6) = ColourT(3, 4, "M_L", "ERR_M_L", "")

    !*****************
    ! These are UV-OPT
    !*****************
    Colour(7)  = ColourT(2, 5, "S_U", "ERR_S_U", "")
    Colour(8)  = ColourT(2, 6, "S_B", "ERR_S_B", "")
    Colour(9)  = ColourT(2, 7, "S_V", "ERR_S_W", "")
    Colour(10) = ColourT(3, 5, "M_U", "ERR_M_U", "")
    Colour(11) = ColourT(3, 6, "M_B", "ERR_M_B", "")
    Colour(12) = ColourT(3, 7, "M_V", "ERR_M_V", "")
    Colour(13) = ColourT(4, 5, "L_U", "ERR_L_U", "")
    Colour(14) = ColourT(4, 6, "L_B", "ERR_L_B", "")
    Colour(15) = ColourT(4, 7, "L_V", "ERR_L_V", "")

end subroutine SetUpInstrumentalColourTable
!******************************************************
! subroutine SetUpStandardColourTable
! Initialises the array with the possible colours
! Letters refer to letter codes for filters
! ie V=V, U=U, B=B, W=White, G=Grism2, X=UVW1,
!   Y=UVM2, Z=UVW2 & H=Grism1
!******************************************************
subroutine SetUpStandardColourTable(Colour)

    type(ColourT), intent(out), dimension(:) :: Colour
     
    Colour(1) = ColourT(5, 6, "U_B", "ERR_U_B", "_U_B")
    Colour(2) = ColourT(6, 7, "B_V", "ERR_B_V", "_B_V")
    Colour(3) = ColourT(5, 7, "U_V", "ERR_U_V", "_U_V")
    Colour(4) = ColourT(4, 5, "L_U", "ERR_L_U", "_L_U")
    Colour(5) = ColourT(4, 6, "L_B", "ERR_L_B", "_L_B")
    Colour(6) = ColourT(4, 7, "L_V", "ERR_L_V", "_L_V")
    Colour(7) = ColourT(2, 6, "S_B", "ERR_S_B", "_S_B")
    Colour(8) = ColourT(2, 7, "S_V", "ERR_S_V", "_S_V")


end subroutine SetUpStandardColourTable
 
!*******************************************************
! CompareDates - Compares the first date with the second.
! Returns -1 if the second is less than the first,
! 0 if they are equal and 1 if greater.
!*******************************************************
integer function CompareDates(dateString1, dateString2)
    character(len = *), intent(in) :: dateString1, dateString2
    integer :: time1(6), time2(6)        ! Stores the time from the date string
    character(len = 1) char    ! Used for date and time extraction 
    integer i

    read(dateString1, '(i4, a1, i2, a1, i2, a1, i2, a1, i2, a1, i2)') &
         time1(1), char, time1(2), char, time1(3), char, time1(4), &
         char, time1(5), char, time1(6)

    read(dateString2, '(i4, a1, i2, a1, i2, a1, i2, a1, i2, a1, i2)') &
         time2(1), char, time2(2), char, time2(3), char, time2(4), &
         char, time2(5), char, time2(6)
  
    do i = 1, 6

        CompareDates = time2(i) - time1(i)
        if (CompareDates .ne. 0) return
 
    end do

    CompareDates = 0 ! Must be equal

end function CompareDates
end module omsrclistcomb_mod




















